#include <SPI.h>
#include <Adafruit_GFX.h>
#include <Max72xxPanel.h>
#include <ESP8266WiFi.h>
#include <PubSubClient.h>

// Belegung
//DOT Matrix:       ESP8266 NodeMCU:
//VCC               5V (VUSB)
//GND               GND
//DIN               D7 (GPIO13)
//CS                D3 (GPIO0)
//CLK               D5 (GPIO14)

int pinCS = 0;                           
int numberOfHorizontalDisplays = 8;     //Anzahl der Module Horizontal
int numberOfVerticalDisplays = 1;        //Anzahl der Module Vertikal

const char* ssid = "SSID WLAN";
const char* password = "WLAN PASSWORT";
const char* mqtt_server = "IP.des.MQTT.Servers";

WiFiClient espClient;
PubSubClient client(espClient);

Max72xxPanel matrix = Max72xxPanel(pinCS, numberOfHorizontalDisplays, numberOfVerticalDisplays);

int ScrollWait = 25;                  // Zeit in ms für Scroll Geschwindigkeit
int helligkeit = 5;             // Helligkeit des DisplaysDefault Helligkeit 0 bis 15
int spacer = 1;                 // Länge eines Leerzeichens
int width = 5 + spacer;         // Schriftgröße
String MatrixText = "Warte auf MQTT Message...";

void setup_wifi() {
  delay(10);
  // Wifi Verbindung
  Serial.println();
  Serial.print("Verbinde mit ");
  Serial.println(ssid);

  WiFi.begin(ssid, password);

  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }

  randomSeed(micros());

  Serial.println("");
  Serial.println("WiFi verbunden :)");
  Serial.println("IP Addresse: ");
  Serial.println(WiFi.localIP());
}

void MQTTCallback(char* topic, byte* payload, unsigned int length) {
  String PayloadString = "";
  for (int i = 0; i < length; i++) { PayloadString = PayloadString + (char)payload[i]; }

  Serial.println("Eingehender Befehl");  
  Serial.println(topic);  
  Serial.println(PayloadString);  
  
  if(strcmp(topic, "LEDMatrix/text") == 0) {  
    Serial.println("Neuer Text wurde gesetzt");      
    MatrixText = PayloadString;
  } 
 
  if(strcmp(topic, "LEDMatrix/intensity") == 0) { 
    Serial.println("Neue Helligkeit wurde gesetzt");      
    helligkeit = PayloadString.toInt();
    matrix.setIntensity(helligkeit);
  }  
  
  if(strcmp(topic, "LEDMatrix/scrollwait") == 0) { 
    Serial.println("Neue Scrollgeschwindigkeit wurde gesetzt");      
    ScrollWait = PayloadString.toInt();
  }  
}

void reconnect() {
  while (!client.connected()) {
    // Client ID generieren
    String clientId = "LEDMatrix-";
    clientId += String(random(0xffff), HEX);

    // Verbindung
    if (client.connect(clientId.c_str())) {
      Serial.println("Mit MQTT verbunden :)"); 
      client.subscribe("LEDMatrix/text");  
      client.subscribe("LEDMatrix/intensity");  
      client.subscribe("LEDMatrix/scrollwait");      
    }
    else
    {
      Serial.print("Fehler, :(");
      Serial.print(client.state());
      Serial.println(" neuer Versuch in 2 Sekunden");
     
      delay(5000);
    }
  }
}

void setup() {
  //delay(1000);

  matrix.setIntensity(helligkeit);
  for (int matrixIndex=0 ; matrixIndex < numberOfHorizontalDisplays ; matrixIndex++ )
  {
    matrix.setRotation(matrixIndex, 1);        //Erste DOT Matrix Drehen 
  }

  Serial.begin(115200);
  Serial.println("Starte LEDMatrix...");

  setup_wifi();
  client.setServer(mqtt_server, 1883);
  client.setCallback(MQTTCallback);  
}

void printMatrix() {
  for ( int i = 0 ; i < width * MatrixText.length() + matrix.width() - 1 - spacer; i++ ) {

    matrix.fillScreen(LOW);

    int letter = i / width;
    int x = (matrix.width() - 1) - i % width;
    int y = (matrix.height() - 8) / 2; //Zentrieren des Textes Vertikal

    while ( x + width - spacer >= 0 && letter >= 0 ) {
      if ( letter < MatrixText.length() ) {
        matrix.drawChar(x, y, MatrixText[letter], HIGH, LOW, 1);
      }
      letter--;
      x -= width;

      client.loop(); 
    }

    matrix.write();
    delay(ScrollWait);
  }
}

void loop() {
  if (!client.connected())  {
    reconnect();
  }
  
  client.loop();    
  delay(100);
    
  printMatrix();      
}
